package com.winhc.repal.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.winhc.common.enums.CodeMsg;
import com.winhc.common.exception.CommonException;
import com.winhc.repal.entity.RepalBill;
import com.winhc.repal.entity.RepalBook;
import com.winhc.repal.enums.DeletedStatusEnum;
import com.winhc.repal.enums.PermissionEnum;
import com.winhc.repal.model.bo.BookDynamicCountBO;
import com.winhc.repal.model.bo.UserBillUnreadInfoBO;
import com.winhc.repal.model.dto.DeleteRepalBookDTO;
import com.winhc.repal.model.dto.HomeTypeDTO;
import com.winhc.repal.model.vo.HomeGradeVO;
import com.winhc.repal.model.vo.RepalBillRemindUnReadVO;
import com.winhc.repal.model.vo.RepalBookInfoVO;
import com.winhc.repal.repository.RepalBookMapper;
import com.winhc.repal.service.*;
import com.winhc.repal.util.BillGradeUtil;
import com.winhc.repal.util.RemindTypeUtil;
import com.winhc.repal.util.UserContextUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @description RepalBook 接口实现类
 * @author Generator
 * @date 2022-04-08
 */
@Service
public class RepalBookServiceImpl extends ServiceImpl<RepalBookMapper, RepalBook> implements RepalBookService {

    @Autowired
    private RepalBookMapper repalBookMapper;

    @Autowired
    private RepalBillService repalBillService;

    @Autowired
    private RepalRolePermissionService repalRolePermissionService;

    @Autowired
    private RepalGroupMemberService repalGroupMemberService;

    @Autowired
    private RepalRemindHistoryService repalRemindHistoryService;

    @Autowired
    private RepalRemindReadInfoService repalRemindReadInfoService;

    @Autowired
    private RepalResponsiblePersonService repalResponsiblePersonService;


    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean deleteRepalBook(DeleteRepalBookDTO dto) {
        long groupId = repalGroupMemberService.getUserGroupId(UserContextUtil.getUser().getUserId());
        int count = this.count(Wrappers.lambdaQuery(RepalBook.class)
                .eq(RepalBook::getRepalGroupId, groupId)
                .eq(RepalBook::getDeleted, DeletedStatusEnum.NORMAL.getCode()));
        if (count < 2) {
            throw new CommonException(CodeMsg.FAILED, "必须保留一个账本哦～");
        }
        if (!this.update(Wrappers.lambdaUpdate(RepalBook.class)
                .eq(RepalBook::getRepalGroupId, groupId)
                .set(RepalBook::getDeleted, DeletedStatusEnum.NORMAL.getCode()))) {
            throw new CommonException(CodeMsg.FAILED, "参数错误");
        }
        // 账款和账款关联责任人也删掉
        repalBillService.deleteRepalBillByBookId(dto.getRepalBookId());
        return true;
    }

    @Override
    public List<RepalBookInfoVO> getRepalBookInfoList() {
        Set<String> codeSet = repalRolePermissionService.getUserPermissionCodeSet(UserContextUtil.getUser().getUserId());
        boolean receivableFlag = codeSet.contains(PermissionEnum.QUERY_ALL_RECEIVABLE.getCode());
        boolean receiptFlag = codeSet.contains(PermissionEnum.QUERY_ALL_RECEIPT.getCode());
        return repalBookMapper.getRepalBookInfoList(UserContextUtil.getUser().getUserId(), repalGroupMemberService.getUserGroupId(UserContextUtil.getUser().getUserId()), receivableFlag, receiptFlag);
    }

    @Override
    public List<List<Integer>> getRepalBookDynamic(HomeTypeDTO dto) {
        List<List<Integer>> resultList = new ArrayList<>();
        long userId = UserContextUtil.getUser().getUserId();
        boolean allFlag = repalRolePermissionService.checkUserPermission(UserContextUtil.getUser().getUserId(), PermissionEnum.QUERY_ALL_RECEIVABLE.getCode());
        // 所有
        LocalDateTime start = null;
        LocalDateTime end = null;
        if (Integer.valueOf(1).equals(dto.getQueryType())) {
            LocalDate now = LocalDate.now();
            start = LocalDateTime.of(now.minusDays(7), LocalTime.MIN);
            end = LocalDateTime.of(now, LocalTime.MAX);
        }
        if (Integer.valueOf(2).equals(dto.getQueryType())) {
            LocalDate now = LocalDate.now();
            start = LocalDateTime.of(now.minusDays(30), LocalTime.MIN);
            end = LocalDateTime.of(now, LocalTime.MAX);
        }
        List<BookDynamicCountBO> countList = repalRemindHistoryService.getBookDynamicCount(dto.getRepalBookId(), allFlag ? userId : null, start, end);
        List<Integer> totalCountList = RemindTypeUtil.getCountList(CollUtil.isEmpty(countList) ? null : countList.stream().collect(Collectors.toMap(BookDynamicCountBO::getRemindType, BookDynamicCountBO::getTotalCount)));
        resultList.add(totalCountList);
        // 未读， 有可能是账本下所有未读
        List<Long> billIdList = repalBillService.getBillIdListByBookId(dto.getRepalBookId(), allFlag ? null : userId);;
        RepalBillRemindUnReadVO unReadVO = repalRemindHistoryService.getRemindUnReadVO(billIdList.stream().map(t -> new RepalBill().setId(t).setRepalBookId(dto.getRepalBookId())).collect(Collectors.toList()), userId, start, end);
        List<Integer> unreadList = new ArrayList<Integer>(){{
            add(unReadVO.getUnreadRiskCount());
            add(unReadVO.getUnreadFinanceCount());
            add(unReadVO.getUnreadRankCount());
            add(unReadVO.getUnreadOverdueCount());
        }};
        resultList.add(unreadList);
        return resultList;
    }

    @Override
    public List<HomeGradeVO> getGradeRateList(HomeTypeDTO dto) {
        boolean allFlag = repalRolePermissionService.checkUserPermission(UserContextUtil.getUser().getUserId(), PermissionEnum.QUERY_ALL_RECEIVABLE.getCode());
        // 所有
        LocalDateTime start = null;
        LocalDateTime end = null;
        if (Integer.valueOf(1).equals(dto.getQueryType())) {
            LocalDate now = LocalDate.now();
            start = LocalDateTime.of(now.minusDays(7), LocalTime.MIN);
            end = LocalDateTime.of(now, LocalTime.MAX);
        }
        if (Integer.valueOf(2).equals(dto.getQueryType())) {
            LocalDate now = LocalDate.now();
            start = LocalDateTime.of(now.minusDays(30), LocalTime.MIN);
            end = LocalDateTime.of(now, LocalTime.MAX);
        }
        List<String> gradeList = repalBillService.getBillGradeByBookId(dto.getRepalBookId(), allFlag ? null : UserContextUtil.getUser().getUserId(), start, end);
        if (CollUtil.isEmpty(gradeList)) {
            return new ArrayList<>();
        }
        return BillGradeUtil.getGradeList(gradeList);
    }
}
